/*
 * Copyright (c) 2023 The Humble Transistor
 * Author: Rob Helvestine <rob@intercreate.io>
 *
 * @file TI LP5030 LED Driver
 *
 */

#pragma once

#include <stdint.h>
#include <stdbool.h>

#include <zephyr/kernel.h>
#include <zephyr/device.h>

#define LP5030_MAX_COMMS_SIZE 0x38

enum lp5030_led {
    NO_LED   = 0,
    LED0     = BIT(0),
    LED1     = BIT(1),
    LED2     = BIT(2),
    LED3     = BIT(3),
    LED4     = BIT(4),
    LED5     = BIT(5),
    LED6     = BIT(6),
    LED7     = BIT(7),
    LED8     = BIT(8),
    LED9     = BIT(9),
    ALL_LEDS = 0x3ff,
};

enum lp5030_control_mode {
    INDEPENDENT = 0,
    BANK        = 1,
};

enum lp5030_color {
    RED,
    GREEN,
    BLUE,
};

struct lp5030_api {
    bool (*write)(const struct device *dev, const uint8_t offset,
            const uint8_t *data, const uint8_t length);
    bool (*read)(const struct device *dev, const uint8_t offset,
            uint8_t *data, const uint8_t length);
    void (*enable)(const struct device *dev, uint8_t val);
    void (*control)(const struct device *dev, uint16_t led_bitmask, 
            enum lp5030_control_mode mode);
    void (*bank_color)(const struct device *dev, enum lp5030_color color, 
            const uint8_t mix);
};

ALWAYS_INLINE static inline bool LP5030_Write(const struct device *dev,
        const uint8_t offset, const uint8_t *data, const uint8_t length)
{
    const struct lp5030_api *api =
        (const struct lp5030_api *)dev->api;
    return api->write(dev, offset, data, length);
}

ALWAYS_INLINE static inline bool LP5030_Read(const struct device *dev,
        const uint8_t offset, uint8_t *data, const uint8_t length)
{
    const struct lp5030_api *api =
        (const struct lp5030_api *)dev->api;
    return api->read(dev, offset, data, length);
}

ALWAYS_INLINE static inline void LP5030_Enable(const struct device *dev,
        uint8_t val)
{
    const struct lp5030_api *api =
        (const struct lp5030_api *)dev->api;
    api->enable(dev, val);
}

ALWAYS_INLINE static inline void LP5030_Control(const struct device *dev,
        uint16_t led_bitmask, enum lp5030_control_mode mode)
{
    const struct lp5030_api *api =
        (const struct lp5030_api *)dev->api;
    api->control(dev, led_bitmask, mode);
}

ALWAYS_INLINE static inline void LP5030_Bank_Color(const struct device *dev,
        enum lp5030_color color, const uint8_t mix)
{
    const struct lp5030_api *api =
        (const struct lp5030_api *)dev->api;
    api->bank_color(dev, color, mix);
}

